@@ -6,6 +6,8 @@ import plotly.express as px
 from itertools import cycle
 palette = cycle(px.colors.qualitative.Plotly)
 
+import numpy as np
+
 
 def create_ohlc_figure(data_df, underlying):
     fig = go.Figure(data=go.Ohlc(x=data_df.index,
@@ -106,11 +108,13 @@ def plot_regime_close(data_df):
 
 
 
-def plot_multiple_time_series(data_df, split =False, logy = True, title = 'Strategy performances', height=600 ,width=1600):
+def plot_multiple_time_series(data_df, split =False, logy = True, title = 'Strategy performances', height=600 ,width=1600, put_on_same_scale = True, drop_na_inf = True):
     palette = cycle(px.colors.qualitative.Bold)
 #    palette = cycle(px.colors.qualitative.Vivid)
 #    palette = cycle(px.colors.qualitative.Safe)
-
+#    data_df[~data_df.isin([np.nan, np.inf, -np.inf]).any(1)] = 0.
+    if drop_na_inf:
+        data_df.iloc[data_df.isin([np.nan, np.inf, -np.inf]).values] = 0.
     if split:
         fig = make_subplots(rows=1, cols=len(data_df.columns))
         constituents = [col for col in data_df.columns if col not in ['date','Date']]
@@ -132,6 +136,8 @@ def plot_multiple_time_series(data_df, split =False, logy = True, title = 'Strat
 
         fig.update_layout(height=height, width=width, title_text=title,showlegend=True)
         fig.update_xaxes(rangeslider_visible=True)
+        if put_on_same_scale:
+            fig.update_yaxes(range=[data_df[constituents].min().min(), data_df[constituents].max().max()])
 
 
         return fig
@@ -154,6 +160,8 @@ def plot_multiple_time_series(data_df, split =False, logy = True, title = 'Strat
             fig.update_yaxes(title_text="y-axis in logarithmic scale", type="log", row=1, col=1)
         fig.update_layout(height=height, width=width, title_text=title,showlegend=True)
         fig.update_xaxes(rangeslider_visible=True)
+        if put_on_same_scale:
+            fig.update_yaxes(range=[data_df[constituents].min().min(), data_df[constituents].max().max()])
 
         return fig
 
