@@ -59,6 +59,153 @@ def lo_slope_ma(data = None, pente_window = 7, contravariant = -1., **kwargs):
     else:
         return 0
 
+def lo_slope_ma_lag(data = None, lag = 7, **kwargs):
+    mma = data.close.mean()
+    y = data.close[-lag:].values
+    slope = stats.linregress(np.arange(lag), y).slope
+    last_value = y[-1]
+    if last_value > mma and slope > 0:
+        return 1
+    else:
+        return 0
+
+def lo_slope_ma_lag_pente_vol(data = None, contravariant = -1., lag = 2, pente_window = 7, ret_threshold = 0.8, **kwargs):
+
+    data['vol'] = np.sqrt(np.log(data['high'] / data['low']) * np.log(data['high'] / data['low']))
+    data['ret'] = (data['close']-data['open'])/data['open']
+    ## aggregating the volumes by lag
+    data.index = range(0,len(data))
+    all_indices = list(reversed(range(len(data) - 1, 0, -lag)))
+    data['aggregated_indice'] = np.nan
+    data['aggregated_indice'].loc[all_indices] = all_indices
+    data['aggregated_indice']= data['aggregated_indice'].fillna(method='bfill')
+    data['aggregated_indice'] = data['aggregated_indice'].astype(int)
+    volume = data.groupby(['aggregated_indice'])['volume'].agg('mean')
+    high = data.groupby(['aggregated_indice'])['high'].agg('max')
+    low = data.groupby(['aggregated_indice'])['low'].agg('min')
+    vol = data.groupby(['aggregated_indice'])['vol'].agg('mean')
+    ret = data.groupby(['aggregated_indice'])['ret'].agg('mean')
+    open = data.groupby(['aggregated_indice'])['open'].first()
+    close = data.groupby(['aggregated_indice'])['close'].last()
+
+    data['aggregated_volume'] =  data['aggregated_indice'].map(volume).copy()
+    data['aggregated_high'] =  data['aggregated_indice'].map(high).copy()
+    data['aggregated_low'] =  data['aggregated_indice'].map(low).copy()
+    data['aggregated_vol'] =  data['aggregated_indice'].map(vol).copy()
+    data['aggregated_ret'] =  data['aggregated_indice'].map(ret).copy()
+
+    data['aggregated_open'] =  data['aggregated_indice'].map(open).copy()
+    data['aggregated_close'] =  data['aggregated_indice'].map(close).copy()
+
+    fractal_df = data.set_index('aggregated_indice')[['aggregated_volume', 'aggregated_high', 'aggregated_low', 'aggregated_vol',
+                                         'aggregated_ret', 'aggregated_close', 'aggregated_open']].drop_duplicates().copy()
+
+    fractal_df = fractal_df.rename(columns = {
+        'aggregated_volume':'volume',
+        'aggregated_high':'high',
+        'aggregated_low':'low',
+        'aggregated_vol':'vol',
+        'aggregated_ret': 'ret',
+        'aggregated_close': 'close',
+        'aggregated_open': 'open'
+    })
+
+    # mma = fractal_df.close.mean()
+    #
+    # y = fractal_df.close[-pente_window:].values
+    # slope = stats.linregress(np.arange(len(y)), y).slope
+    # last_value = y[-1]
+    # gen_sig = np.nan
+    # if last_value > mma and slope > 0:
+    #     gen_sig= 1
+    # else:
+    #     gen_sig= 0
+
+    def compute_slope(slope_df):
+        y = slope_df.values
+        slope = stats.linregress(np.arange(len(y)), y).slope
+        return slope
+
+    fractal_df['rolling_slope'] = fractal_df['close'].rolling(window=pente_window).apply(compute_slope)
+
+    fractal_df['rolling_slope_rank'] = fractal_df['rolling_slope'].rank(pct=True)
+
+    gen_sig = fractal_df['rolling_slope_rank'].iloc[-1]
+# #    fractal_df['open_ma_lagged'] = fractal_df['open_ma'].shift(instantaneous_slope_fin_diff_h)
+# #    fractal_df['open_ma_slope'] = (fractal_df['open_ma'] - data_df['open_ma_lagged']) / instantaneous_slope_fin_diff_h
+#     fractal_df['trend_signal'] = fractal_df['rolling_slope'] > 0
+#     fractal_df['no_trend_signal'] = fractal_df['rolling_slope'] <= 0
+#     fractal_df['trend_signal_mastd'] = fractal_df['trend_signal'].rolling(window=pente_window).std()
+#     fractal_df['trend_signal_mastd_ma'] = fractal_df['trend_signal_mastd'].rolling(window=pente_window).std()
+#     fractal_df['trend_signal_ma'] = fractal_df['trend_signal'].rolling(window=pente_window).mean()
+#     fractal_df['moderator_signal'] = fractal_df['trend_signal_mastd_ma'] <= 0.5
+#     fractal_df['signal'] = fractal_df['signal'] * fractal_df['moderator_signal']
+
+    return gen_sig
+
+def lo_slope_ma_lag_fractal(data = None, contravariant = -1., lag = 2, pente_window = 7, ret_threshold = 0.8, **kwargs):
+    # data_df['open_ma'] = data_df['open'].rolling(window=rolling_window).mean()
+    # data_df['open_ma_lagged'] = data_df['open_ma'].shift(instantaneous_slope_fin_diff_h)
+    # data_df['open_ma_slope'] = (data_df['open_ma'] - data_df['open_ma_lagged']) / instantaneous_slope_fin_diff_h
+    # data_df['trend_signal'] = data_df['open_ma_slope'] > 0
+    # data_df['no_trend_signal'] = data_df['open_ma_slope'] <= 0
+    # data_df['trend_signal_mastd'] = data_df['trend_signal'].rolling(window=signal_vol_window).std()
+    # data_df['trend_signal_mastd_ma'] = data_df['trend_signal_mastd'].rolling(window=signal_vol_window).std()
+    # data_df['trend_signal_ma'] = data_df['trend_signal'].rolling(window=signal_vol_window).mean()
+    # data_df['moderator_signal'] = data_df['trend_signal_mastd_ma'] <= threshold
+    # data_df['signal'] = data_df['signal'] * data_df['moderator_signal']
+
+
+    data['vol'] = np.sqrt(np.log(data['high'] / data['low']) * np.log(data['high'] / data['low']))
+    data['ret'] = (data['close']-data['open'])/data['open']
+    ## aggregating the volumes by lag
+    data.index = range(0,len(data))
+    all_indices = list(reversed(range(len(data) - 1, 0, -lag)))
+    data['aggregated_indice'] = np.nan
+    data['aggregated_indice'].loc[all_indices] = all_indices
+    data['aggregated_indice']= data['aggregated_indice'].fillna(method='bfill')
+    data['aggregated_indice'] = data['aggregated_indice'].astype(int)
+    volume = data.groupby(['aggregated_indice'])['volume'].agg('mean')
+    high = data.groupby(['aggregated_indice'])['high'].agg('max')
+    low = data.groupby(['aggregated_indice'])['low'].agg('min')
+    vol = data.groupby(['aggregated_indice'])['vol'].agg('mean')
+    ret = data.groupby(['aggregated_indice'])['ret'].agg('mean')
+    open = data.groupby(['aggregated_indice'])['open'].first()
+    close = data.groupby(['aggregated_indice'])['close'].last()
+
+    data['aggregated_volume'] =  data['aggregated_indice'].map(volume).copy()
+    data['aggregated_high'] =  data['aggregated_indice'].map(high).copy()
+    data['aggregated_low'] =  data['aggregated_indice'].map(low).copy()
+    data['aggregated_vol'] =  data['aggregated_indice'].map(vol).copy()
+    data['aggregated_ret'] =  data['aggregated_indice'].map(ret).copy()
+
+    data['aggregated_open'] =  data['aggregated_indice'].map(open).copy()
+    data['aggregated_close'] =  data['aggregated_indice'].map(close).copy()
+
+    fractal_df = data.set_index('aggregated_indice')[['aggregated_volume', 'aggregated_high', 'aggregated_low', 'aggregated_vol',
+                                         'aggregated_ret', 'aggregated_close', 'aggregated_open']].drop_duplicates().copy()
+
+    fractal_df = fractal_df.rename(columns = {
+        'aggregated_volume':'volume',
+        'aggregated_high':'high',
+        'aggregated_low':'low',
+        'aggregated_vol':'vol',
+        'aggregated_ret': 'ret',
+        'aggregated_close': 'close',
+        'aggregated_open': 'open'
+    })
+
+    mma = fractal_df.close.mean()
+
+    y = fractal_df.close[-pente_window:].values
+    slope = stats.linregress(np.arange(len(y)), y).slope
+    last_value = y[-1]
+    if last_value > mma and slope > 0:
+        return 1
+    else:
+        return 0
+
+
 def alpha_2(data = None,  contravariant = -1., **kwargs):
     data['log_vol'] = np.log(data['volume'])
     correlation_coefficient = compute_correlation(data.log_vol.diff(2).rank(pct=True), (data.close - data.open)/data.open)
@@ -797,7 +944,7 @@ def trending_rank_lo_induced(data = None, contravariant = -1., up_trend_threshol
     data['aggregated_indice'].loc[all_indices] = all_indices
     data['aggregated_indice']= data['aggregated_indice'].fillna(method='bfill')
     data['aggregated_indice'] = data['aggregated_indice'].astype(int)
-    volume = data.groupby(['aggregated_indice'])['volume'].agg('sum')
+    volume = data.groupby(['aggregated_indice'])['volume'].agg('mean')
     high = data.groupby(['aggregated_indice'])['high'].agg('max')
     low = data.groupby(['aggregated_indice'])['low'].agg('min')
 
@@ -1144,6 +1291,88 @@ def counting_candles_ls(data= None, low_threshold = 0.3, up_threshold = 0.8, con
     else:
         return np.nan
 
+def test_sig(data = None, contravariant = -1., lag = 2, vol_threshold = 0.1, ret_threshold = 0.8, **kwargs):
+    # data_df['open_ma'] = data_df['open'].rolling(window=rolling_window).mean()
+    # data_df['open_ma_lagged'] = data_df['open_ma'].shift(instantaneous_slope_fin_diff_h)
+    # data_df['open_ma_slope'] = (data_df['open_ma'] - data_df['open_ma_lagged']) / instantaneous_slope_fin_diff_h
+    # data_df['trend_signal'] = data_df['open_ma_slope'] > 0
+    # data_df['no_trend_signal'] = data_df['open_ma_slope'] <= 0
+    # data_df['trend_signal_mastd'] = data_df['trend_signal'].rolling(window=signal_vol_window).std()
+    # data_df['trend_signal_mastd_ma'] = data_df['trend_signal_mastd'].rolling(window=signal_vol_window).std()
+    # data_df['trend_signal_ma'] = data_df['trend_signal'].rolling(window=signal_vol_window).mean()
+    # data_df['moderator_signal'] = data_df['trend_signal_mastd_ma'] <= threshold
+    # data_df['signal'] = data_df['signal'] * data_df['moderator_signal']
+
+
+    data['vol'] = np.sqrt(np.log(data['high'] / data['low']) * np.log(data['high'] / data['low']))
+    data['ret'] = (data['close']-data['open'])/data['open']
+    ## aggregating the volumes by lag
+    data.index = range(0,len(data))
+    all_indices = list(reversed(range(len(data) - 1, 0, -lag)))
+    data['aggregated_indice'] = np.nan
+    data['aggregated_indice'].loc[all_indices] = all_indices
+    data['aggregated_indice']= data['aggregated_indice'].fillna(method='bfill')
+    data['aggregated_indice'] = data['aggregated_indice'].astype(int)
+    volume = data.groupby(['aggregated_indice'])['volume'].agg('mean')
+    high = data.groupby(['aggregated_indice'])['high'].agg('max')
+    low = data.groupby(['aggregated_indice'])['low'].agg('min')
+    vol = data.groupby(['aggregated_indice'])['vol'].agg('mean')
+    ret = data.groupby(['aggregated_indice'])['ret'].agg('mean')
+    open = data.groupby(['aggregated_indice'])['open'].first()
+    close = data.groupby(['aggregated_indice'])['close'].last()
+
+    data['aggregated_volume'] =  data['aggregated_indice'].map(volume).copy()
+    data['aggregated_high'] =  data['aggregated_indice'].map(high).copy()
+    data['aggregated_low'] =  data['aggregated_indice'].map(low).copy()
+    data['aggregated_vol'] =  data['aggregated_indice'].map(vol).copy()
+    data['aggregated_ret'] =  data['aggregated_indice'].map(ret).copy()
+
+    data['aggregated_open'] =  data['aggregated_indice'].map(open).copy()
+    data['aggregated_close'] =  data['aggregated_indice'].map(close).copy()
+
+    fractal_df = data.set_index('aggregated_indice')[['aggregated_volume', 'aggregated_high', 'aggregated_low', 'aggregated_vol',
+                                         'aggregated_ret', 'aggregated_close', 'aggregated_open']].drop_duplicates().copy()
+
+    fractal_df = fractal_df.rename(columns = {
+        'aggregated_volume':'volume',
+        'aggregated_high':'high',
+        'aggregated_low':'low',
+        'aggregated_vol':'vol',
+        'aggregated_ret': 'ret',
+        'aggregated_close': 'close',
+        'aggregated_open': 'open'
+    })
+
+    mma = fractal_df.close.mean()
+    y = fractal_df.close[-lag:].values
+    slope = stats.linregress(np.arange(lag), y).slope
+    last_value = y[-1]
+    if last_value > mma and slope > 0:
+        return 1
+    else:
+        return 0
+
+
+
+def simple_counting_candles_lo(data= None, up_threshold = 0.8, **kwargs):
+    data['close_diff'] = ((data['close']-data['open'])>= 0.).astype(float)
+    ratio_rank = data['close_diff'].mean()
+    if ratio_rank >= up_threshold:
+        return 1
+    else:
+        return np.nan
+
+def simple_counting_candles_ls(data=None, up_threshold=0.8, down_threshold=0.2, **kwargs):
+    data['close_diff'] = ((data['close'] - data['open']) >= 0.).astype(float)
+    ratio_rank = data['close_diff'].mean()
+    if ratio_rank >= up_threshold:
+        return 1
+    elif ratio_rank <= down_threshold:
+        return -1
+    else:
+        return np.nan
+
+
 def counting_candles_lo(data= None, up_threshold = 0.8, contravariant = -1., lag = 2, **kwargs):
     data['close_diff']= (data.close.diff() >= 0.).astype(float)
 
@@ -1169,9 +1398,9 @@ def counting_candles_lo(data= None, up_threshold = 0.8, contravariant = -1., lag
 
 
     data['positive_candle_pct_rank'] = data['aggregated_close_diff'].rank(pct=True)
-
     ratio_rank = data['positive_candle_pct_rank'].iloc[-1]
 
+
     if ratio_rank >= up_threshold:
         return contravariant
     else:
