@@ -73,7 +73,7 @@ def accuracy(y_true, y_pred, sign=True):
     return R / (R + W)
 
 
-def annual_return(series, period=252):
+def annual_return(series, period=252,display = True):
     """ Compute compouned annual return.
 
     Parameters
@@ -103,6 +103,11 @@ def annual_return(series, period=252):
     """
     T = series.size
     ret = series[-1] / series[0]
+    if display:
+        print(f'series size {T}')
+        print(f'period {period}')
+        print(f'series[-1] {series[-1]}')
+        print(f'series[0] {series[0]}')
 
     return np.sign(ret) * np.float_power(
         np.abs(ret),
